@@ -111,9 +111,10 @@ def procrustes_init(seq, rho, y, is_visited_view, d, Utilde, n_Utilde_Utilde,
     return y, is_visited_view
     
 # Ngoc-Diep Ho, Paul Van Dooren, On the pseudo-inverse of the Laplacian of a bipartite graph
-def compute_Lpinv_BT(Utilde, B):
-    M, n = Utilde.shape
-    B_ = Utilde.copy().transpose().astype('int')
+def compute_Lpinv_BT(W, B):
+    M, n = W.shape
+    # B_ = W.copy().transpose().astype('int')
+    B_ = W.copy().transpose().astype('float')
     D_1 = np.asarray(B_.sum(axis=1))
     D_2 = np.asarray(B_.sum(axis=0))
     D_1_inv_sqrt = np.sqrt(1/D_1)
@@ -175,37 +176,88 @@ def compute_CC(D, B, Lpinv_BT):
     CC = D - B.dot(Lpinv_BT)
     return 0.5*(CC + CC.T)
 
-def build_ortho_optim(d, Utilde, intermed_param, ret_D=False):
+def build_ortho_optim(d, Utilde, intermed_param, ret_D=False,
+                      far_off_points=[], repel_by=0.,
+                      beta=None):
     M,n = Utilde.shape
     B_row_inds = []
     B_col_inds = []
     B_vals = []
     D = []
+    
+    W_row_inds = []
+    W_col_inds = []
+    W_vals = []
+    
     for i in range(M):
+        Utilde_i = Utilde[i,:].indices
         X_ = intermed_param.eval_({'view_index': i,
-                                   'data_mask': Utilde[i,:].indices})
+                                   'data_mask': Utilde_i})
+        
+        w = None
+        if beta:
+            anom_scores = intermed_param.anom_score_({'view_index': i, 'data_mask': Utilde_i})
+            if anom_scores is not None:
+                w = np.exp(-anom_scores/beta)
+        
+        if w is None:
+            w = np.ones((Utilde_i.shape[0]))
+        
+        X_ = w[:,None] * X_
+        
         D.append(np.matmul(X_.T,X_))
+        
         row_inds = list(range(d*i,d*(i+1)))
-        col_inds = Utilde[i,:].indices.tolist()
+        col_inds = Utilde_i.tolist()
+        
+        W_row_inds += [i]*len(col_inds)
+        W_col_inds += col_inds
+        W_vals += (w**2).tolist()
+        
         B_row_inds += (row_inds + np.repeat(row_inds, len(col_inds)).tolist())
         B_col_inds += (np.repeat([n+i], d).tolist() + np.tile(col_inds, d).tolist())
         B_vals += (np.sum(-X_.T, axis=1).tolist() + X_.T.flatten().tolist())
     
     D = block_diag(D, format='csr')
     B = csr_matrix((B_vals, (B_row_inds, B_col_inds)), shape=(M*d,n+M))
+    W = csr_matrix((W_vals, (W_row_inds, W_col_inds)), shape=(M,n), dtype=float)
+    
+    if beta:
+        print('min and max anom scores:', (-beta*np.log(np.array(W_vals))).min(),
+                                          (-beta*np.log(np.array(W_vals))).max())
+    print('min and max weights:', np.array(W_vals).min(), np.array(W_vals).max())
 
     print('Computing Pseudoinverse of a matrix of L of size', n, '+', M, 'multiplied with B', flush=True)
-    Lpinv_BT = compute_Lpinv_BT(Utilde, B)
+    Lpinv_BT = compute_Lpinv_BT(W, B)
 
     CC = compute_CC(D, B, Lpinv_BT)
+    
+    n_repel = len(far_off_points)
+    if n_repel > 0:
+        L__row_inds = []
+        L__col_inds = []
+        L__vals = []
+        temp_arr = (-np.ones(n_repel)).tolist()
+        for i in range(n_repel):
+            L__row_inds += [far_off_points[i]]*n_repel
+            L__col_inds += far_off_points
+            temp_arr[i] = n_repel
+            L__vals += temp_arr
+            temp_arr[i] = -1
+        L_ = csr_matrix((L__vals, (L__row_inds, L__col_inds)), shape=(n+M,n+M))
+        L_ = -repel_by*L_
+        CC = CC + (Lpinv_BT.T).dot(L_.dot(Lpinv_BT))
+    
     if ret_D:
         return CC, Lpinv_BT, D
     else:
         return CC, Lpinv_BT
     
 
-def compute_alignment_err(d, Utilde, intermed_param, scale_num):
-    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param)
+def compute_alignment_err(d, Utilde, intermed_param, scale_num, far_off_points=[], repel_by=0., beta=None):
+    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param,
+                                     far_off_points=far_off_points,
+                                     repel_by=repel_by, beta=beta)
     M,n = Utilde.shape
     
     ## Check if C is pd or psd
@@ -225,7 +277,10 @@ def compute_alignment_err(d, Utilde, intermed_param, scale_num):
 def spectral_alignment(y, is_visited_view, d, Utilde,
                       C, intermed_param, global_opts, 
                       seq_of_intermed_views_in_cluster):
-    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param)
+    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param,
+                                     far_off_points=global_opts['far_off_points'],
+                                     repel_by=global_opts['repel_by'],
+                                     beta=global_opts['beta'])
         
     M,n = Utilde.shape
     n_clusters = len(seq_of_intermed_views_in_cluster)
@@ -236,7 +291,7 @@ def spectral_alignment(y, is_visited_view, d, Utilde,
     # To find smallest eigenvalues, using shift-inverted algo with mode=normal and which='LM'
     W_,V_ = scipy.sparse.linalg.eigsh(CC, k=d, v0=v0, sigma=0.0)
     # or just pass which='SM' without using sigma
-    #W_,V_ = scipy.sparse.linalg.eigsh(CC, k=d, v0=v0, which='SM')
+    # W_,V_ = scipy.sparse.linalg.eigsh(CC, k=d, v0=v0, which='SM')
     print('Done.', flush=True)
     Wstar = np.sqrt(M)*V_.T
     Tstar = np.zeros((d, M*d))
@@ -352,7 +407,10 @@ def rgd_final(y, d, Utilde, C, intermed_param,
              parents_of_intermed_views_in_cluster,
              cluster_of_intermed_view,
              global_opts):
-    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param)
+    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param,
+                                     far_off_points=global_opts['far_off_points'],
+                                     repel_by=global_opts['repel_by'],
+                                     beta=global_opts['beta'])
     M,n = Utilde.shape
     n_proc = min(M,global_opts['n_proc'])
     barrier = mp.Barrier(n_proc)
@@ -458,7 +516,10 @@ def gpm_final(y, d, Utilde, C, intermed_param,
              parents_of_intermed_views_in_cluster,
              cluster_of_intermed_view,
              global_opts):
-    CC, Lpinv_BT, D = build_ortho_optim(d, Utilde, intermed_param, ret_D=True)
+    CC, Lpinv_BT, D = build_ortho_optim(d, Utilde, intermed_param, ret_D=True,
+                                        far_off_points=global_opts['far_off_points'],
+                                        repel_by=global_opts['repel_by'],
+                                        beta=global_opts['beta'])
     CC = D - CC
     M,n = Utilde.shape
     n_proc = min(M,global_opts['n_proc'])
@@ -560,7 +621,10 @@ def sdp_alignment(y, is_visited_view, d, Utilde,
                   C, intermed_param, global_opts, 
                   seq_of_intermed_views_in_cluster,
                   solver=None):
-    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param)
+    CC, Lpinv_BT = build_ortho_optim(d, Utilde, intermed_param,
+                                     far_off_points=global_opts['far_off_points'],
+                                     repel_by=global_opts['repel_by'],
+                                     beta=global_opts['beta'])
     M,n = Utilde.shape
     b = vec(CC)
     if solver is None:
