@@ -6,6 +6,7 @@ from scipy.spatial.distance import pdist, squareform
 from sklearn.neighbors import NearestNeighbors, KNeighborsTransformer
 
 from scipy.sparse import csr_matrix
+from scipy.sparse.csgraph import dijkstra
 
 from . import local_views_
 from . import intermed_views_
@@ -179,19 +180,20 @@ def double_manifold_k_nn(data, ddX, k_nn, metric, n_proc=1):
     return neigh_dist, neigh_ind
     
 
-def get_default_local_opts(algo='LDLE', k_nn=49, k_tune=7, k=28, radius=0.5, U_method='k_nn', 
-                           gl_type='unnorm', N=100, scale_by='gamma', Atilde_method='LDLE_1',
-                           alpha=1, max_iter=300, reg=0.,
+def get_default_local_opts(algo='LDLE', k_nn=49, k_tune=7, k=28, metric='euclidean', radius=0.5,
+                           U_method='k_nn', gl_type='unnorm', N=100, scale_by='gamma',
+                           Atilde_method='LDLE_1', alpha=1, max_iter=300, reg=0.,
                            p=0.99, tau=50, delta=0.9, to_postprocess= True,
-                           pp_n_thresh=32):
+                           pp_n_thresh=32, lambda1_init=8, lambda1_decay=0.75, lambda1_min=1e-3,
+                           power=5, max_sparsity=0.9):
     """Sets and returns a dictionary of default_local_opts.
     
     Parameters
     ----------
     algo : str
            The algorithm to use for the construction of
-           local views. Options are 'LDLE', 'LTSA' and
-           'Smooth-LTSA'. LTSA uses the hyperparameter k and 
+           local views. Options are 'LDLE', 'LTSA', 'SparsePCA',
+           and 'RPCA-GODEC'. LTSA uses the hyperparameter k and 
            k_nn only and is not affected by the value of the
            others. Smooth-LTSA additionally uses alpha,
            max_iter, reg. Currently, Smooth-LTSA is very slow.
@@ -203,6 +205,8 @@ def get_default_local_opts(algo='LDLE', k_nn=49, k_tune=7, k=28, radius=0.5, U_m
            be less than k_nn.
     k : int 
         The size of local view per point.
+    metric : str
+             The metric to use for finding nearest neighbors.
     radius : float
              Radius of the balls to be used to find nearest neighbors.
     U_method : str
@@ -237,12 +241,12 @@ def get_default_local_opts(algo='LDLE', k_nn=49, k_tune=7, k=28, radius=0.5, U_m
     delta : float
         A hyperparameter used in computing parameterizations
         of local views. The value must be in (0,1).
-    alpha: float
+    alpha : float
            Step size in Riemannian gradient descent when using
            Smooth-LTSA.
     max_iter: int
               Max number of Riemannian gradient descent steps.
-    reg: float
+    reg : float
          Desired regularization (Smoothness) in Smooth-LTSA.
     to_postprocess : bool
         If True the local parameterizations are postprocessed
@@ -252,13 +256,24 @@ def get_default_local_opts(algo='LDLE', k_nn=49, k_tune=7, k=28, radius=0.5, U_m
         Threshold to use multiple processors or a single processor
         while postprocessing the local parameterizations. A small
         value such as 32 leads to faster postprocessing.
+    lambda1_init : float
+                  Initialization of lambda1 for RPCA-GODEC.
+    lambda1_decay : float
+                   Decay of lambda1 for RPCA-GODEC.
+    lambda1_min : float
+                 Minimum allowed valued of lambda1 for RPCA-GODEC.
+    power : int
+           Number of power iterations for initialization in RPCA-GODEC.
+    max_sparsity : float
+                  maximum fraction of the desired sparsity.
     """
-    return {'k_nn': k_nn, 'k_tune': k_tune, 'k': k, 'radius': radius,
+    return {'k_nn': k_nn, 'k_tune': k_tune, 'k': k, 'metric': metric, 'radius': radius,
            'U_method': U_method, 'gl_type': gl_type, 'N': N, 'scale_by': scale_by,
            'Atilde_method': Atilde_method, 'p': p, 'tau': tau, 'delta': delta,
            'alpha': alpha, 'max_iter': max_iter, 'reg': reg, 
            'to_postprocess': to_postprocess, 'algo': algo,
-           'pp_n_thresh': pp_n_thresh}
+           'pp_n_thresh': pp_n_thresh, 'lambda1_init': lambda1_init, 'lambda1_decay': lambda1_decay,
+           'lambda1_min': lambda1_min, 'power': power, 'max_sparsity': max_sparsity}
 
 def get_default_intermed_opts(algo='best', n_times=4, eta_min=5, eta_max=25, len_S_thresh=256):
     """Sets and returns a dictionary of default_intermed_opts.
@@ -293,9 +308,8 @@ def get_default_global_opts(main_algo='LDLE', to_tear=True, nu=3, max_iter=20, c
                             vis_before_init=False, compute_error=False,
                             init_algo_name='procrustes', align_w_parent_only=True,
                             refine_algo_name='rgd',
-                            max_internal_iter=100,
-                            alpha=0.3,
-                            eps=1e-8):
+                            max_internal_iter=100, alpha=0.3, eps=1e-8,
+                            add_dim=False, beta=None, repel_by=0., n_repel=0):
     """Sets and returns a dictionary of default_global_opts.
 
     Parameters
@@ -351,6 +365,17 @@ def get_default_global_opts(main_algo='LDLE', to_tear=True, nu=3, max_iter=20, c
     eps : float
           The tolerance used by sdp solver when the init or refinement
           algorithm is 'sdp'.
+    add_dim : bool
+             add an extra dimension to intermediate views.
+    beta: float
+          Hyperparameter used for computing the alignment weights. If
+          None, the alignment is not weighted.
+    repel_by : float
+               If positive, the points which are far off are repelled
+              away from each other by a force proportional to it.
+              Ignored when refinement algorithm is 'procrustes'.
+    n_repel : int
+              The number of far off points repelled from each other.
     """
     return {'to_tear': to_tear, 'nu': nu, 'max_iter': max_iter,
                'color_tear': color_tear,
@@ -361,8 +386,8 @@ def get_default_global_opts(main_algo='LDLE', to_tear=True, nu=3, max_iter=20, c
                'align_w_parent_only': align_w_parent_only,
                'refine_algo_name': refine_algo_name, 
                'max_internal_iter': max_internal_iter,
-               'alpha': alpha,
-               'eps': eps
+               'alpha': alpha, 'eps': eps, 'add_dim': add_dim,
+               'beta': beta, 'repel_by': repel_by, 'n_repel': n_repel
               }
 def get_default_vis_opts(save_dir='', cmap_interior='summer', cmap_boundary='jet', c=None):
     """Sets and returns a dictionary of default_vis_opts.
@@ -447,6 +472,7 @@ class LDLE:
         print("local_opts['k_nn0'] =", self.local_opts['k_nn0'], "is created.")
         #############################################
         global_opts['k'] = self.local_opts['k']
+        global_opts['metric'] = self.local_opts['metric']
         global_opts['n_proc'] = n_proc
         global_opts['verbose'] = verbose
         global_opts['debug'] = debug
@@ -529,12 +555,12 @@ class LDLE:
         if ddX is None or self.local_opts['algo'] == 'LTSA':
             neigh_dist, neigh_ind = nearest_neighbors(data,
                                                       k_nn=self.local_opts['k_nn0'],
-                                                      metric='euclidean')
+                                                      metric=self.local_opts['metric'])
         else:
             self.log('Doubling manifold.')
             neigh_dist, neigh_ind = double_manifold_k_nn(data, ddX,
                                                          self.local_opts['k_nn0'],
-                                                         'euclidean',
+                                                         self.local_opts['metric'],
                                                          self.local_opts['n_proc'])
             self.log('Done.', log_time=True)
         
@@ -578,11 +604,27 @@ class LDLE:
         if self.exit_at == 'intermed_views':
             return
         
-        # Construct Global views
+        # Construct Global view
+        # the far off points which are to be repelled from each other
+        np.random.seed(42)
+        far_off_points = []
+        dist_from_far_off_points = None
+        while len(far_off_points) < self.global_opts['n_repel']:
+            if len(far_off_points) == 0:
+                far_off_points = [np.random.randint(0,d_e.shape[0])]
+                dist_from_far_off_points = dijkstra(d_e, directed=False,
+                                                    indices=far_off_points[-1])
+            else:
+                far_off_points.append(np.argmax(dist_from_far_off_points))
+                dist_from_far_off_points = np.minimum(dist_from_far_off_points,
+                                                      dijkstra(d_e, directed=False,
+                                                               indices=far_off_points[-1]))
+        self.global_opts['far_off_points'] = far_off_points
+        # Global view
         GlobalViews = global_views_.GlobalViews(self.exit_at, self.verbose, self.debug)
         GlobalViews.fit(self.d, IntermedViews.Utilde, IntermedViews.C, IntermedViews.c,
-                        IntermedViews.n_C, IntermedViews.intermed_param, self.global_opts,
-                        self.vis, self.vis_opts)
+                        IntermedViews.n_C, IntermedViews.intermed_param,
+                        self.global_opts, self.vis, self.vis_opts)
         
         self.GlobalViews = GlobalViews
         
